@@ -8,6 +8,7 @@ import urllib
 from flask import jsonify
 import shutil
 import string
+import hashlib
 
 
 class Popen(subprocess.Popen):
@@ -133,6 +134,25 @@ def runSoftwareWithLock(config):
     slapgrid = Popen([config['slapgrid_sr'], '-vc', config['configuration_file_path']], stdout=logfile)
     writePid(slapgrid_pid, slapgrid.pid)
     slapgrid.wait()
+    #Saves the current compile software for re-use
+    #This uses the new software create by slapgrid (if not exits yet)
+    data = loadSoftwareData(config['runner_workdir'])
+    md5 = ""    
+    for path in os.listdir(config['software_root']):
+      exist = False
+      for val in data:
+	if val['md5'] == path:
+	  exist = True
+      conf = os.path.join(config['runner_workdir'], ".project")
+      if not exist: #save this compile software folder
+	if os.path.exists(conf):
+	  data.append({"title":getProjectTitle(config), "md5":path,
+	              "path": open(os.path.join(config['runner_workdir'],
+	                                        ".project"), 'r').read()})
+	  writeSoftwareData(config['runner_workdir'], data)
+	else:
+	  shutil.rmtree(os.path.join(config['software_root'], path))
+	break
     return True
   return False
 
@@ -170,7 +190,7 @@ def getProfile(peojectDir, profileName):
   if os.path.exists(profile):
     return open(profile).read()
   else:
-    return ''
+    return None
 
 def getProfilePath(peojectDir, profile):
   if not os.path.exists(os.path.join(peojectDir, ".project")):
@@ -204,9 +224,24 @@ def runBuildoutAnnotate(config):
   return False
 
 def svcStopAll(config):
+  #stop all process running in supervisord
+  request = Popen([config['supervisor'], config['configuration_file_path'], 
+                'stop', 'all'])
+  request.wait()
   return Popen([config['supervisor'], config['configuration_file_path'], 
                 'shutdown']).communicate()[0]
 
+def removeInstanceRoot(config):
+  if os.path.exists(config['instance_root']):
+    svcStopAll(config)
+    for root, dirs, files in os.walk(config['instance_root']):
+      for fname in dirs:
+	fullPath = os.path.join(root, fname)
+	if not os.access(fullPath, os.W_OK) :
+	  # Some directories may be read-only, preventing to remove files in it
+	  os.chmod(fullPath, 0744)
+    shutil.rmtree(config['instance_root'])
+
 def getSvcStatus(config):
   result = Popen([config['supervisor'], config['configuration_file_path'], 
                   'status']).communicate()[0]
@@ -222,7 +257,7 @@ def getSvcTailProcess(config, process):
                 "tail", process]).communicate()[0]
 
 def svcStartStopProcess(config, process, action):
-  cmd = {"RESTART":"restart", "STOPPED":"start", "RUNNING":"stop"}
+  cmd = {"RESTART":"restart", "STOPPED":"start", "RUNNING":"stop", "EXITED":"start", "STOP":"stop"}
   return Popen([config['supervisor'], config['configuration_file_path'], 
                 cmd[action], process]).communicate()[0]
 
@@ -231,7 +266,8 @@ def getFolderContent(folder):
   try:
     r=['<ul class="jqueryFileTree" style="display: none;">']
     d=urllib.unquote(folder)
-    for f in os.listdir(d):
+    ldir = sorted(os.listdir(d), key=unicode.lower)
+    for f in ldir:
       if f.startswith('.'): #do not displays this file/folder
 	continue      
       ff=os.path.join(d,f)
@@ -249,9 +285,10 @@ def getFolderContent(folder):
 def getFolder(folder):
   r=['<ul class="jqueryFileTree" style="display: none;">']
   try:
-    r=['<ul class="jqueryFileTree" style="display: none;">']
+    r=['<ul class="jqueryFileTree" style="display: none;">']    
     d=urllib.unquote(folder)
-    for f in os.listdir(d):
+    ldir = sorted(os.listdir(d), key=unicode.lower)
+    for f in ldir:
       if f.startswith('.'): #do not display this file/folder
 	continue
       ff=os.path.join(d,f)
@@ -265,10 +302,22 @@ def getFolder(folder):
 
 def getProjectList(folder):
   project = []
-  for elt in os.listdir(folder):
+  project_list = sorted(os.listdir(folder), key=str.lower)
+  for elt in project_list:
     project.append(elt)
   return project
 
+def configNewSR(config, project):
+  if os.path.exists(project):
+    stopProxy(config)
+    removeProxyDb(config)
+    startProxy(config)
+    removeInstanceRoot(config)
+    open(os.path.join(config['runner_workdir'], ".project"), 'w').write(project)
+    return True
+  else:
+    return False
+
 def newSoftware(folder, config, session):
   json = ""
   code = 0
@@ -304,8 +353,7 @@ def checkSoftwareFolder(path, config):
   tmp = path.split('/')
   del tmp[len(tmp) - 1]
   path = string.join(tmp, '/')
-  if os.path.exists(os.path.join(path, config['software_profile'])) and \
-     os.path.exists(os.path.join(path, config['instance_profile'])):
+  if os.path.exists(os.path.join(path, config['software_profile'])):
     return jsonify(result=path)
   return jsonify(result="")
 
@@ -315,5 +363,96 @@ def getProjectTitle(config):
     project = open(conf, "r").read().replace(config['workspace'] + "/", "").split("/")
     software = project[len(project) - 1]
     del project[len(project) - 1]
-    return software + "(" + string.join(project, '/') + ")"
-  return ""
\ No newline at end of file
+    return software + "(SR in /" + string.join(project, '/') + ")"
+  return "No Profile"
+
+def loadSoftwareData(runner_dir):
+  import pickle
+  file_path = os.path.join(runner_dir, '.softdata')
+  if not os.path.exists(file_path):
+    return []
+  pkl_file = open(file_path, 'rb')
+  data = pickle.load(pkl_file)
+  pkl_file.close()
+  return data
+
+def writeSoftwareData(runner_dir, data):
+  import pickle
+  file_path = os.path.join(runner_dir, '.softdata')
+  pkl_file = open(file_path, 'wb')
+  # Pickle dictionary using protocol 0.
+  pickle.dump(data, pkl_file)
+  pkl_file.close()
+  
+def removeSoftwareByName(config, folderName):
+  if isSoftwareRunning(config) or isInstanceRunning(config):
+    return jsonify(code=0, result="Software installation or instantiation in progress, cannot remove")
+  path = os.path.join(config['software_root'], folderName)
+  if not os.path.exists(path):
+    return jsonify(code=0, result="Can not remove software: No such file or directory")
+  svcStopAll(config)
+  shutil.rmtree(path)
+  #update compiled software list
+  data = loadSoftwareData(config['runner_workdir'])
+  i = 0
+  for p in data:
+    if p['md5'] == folderName:
+      del data[i]
+      writeSoftwareData(config['runner_workdir'], data)
+      break
+    i = i+1
+  return jsonify(code=1, result=data)
+
+def tail(f, lines=20):
+  """
+  Returns the last `lines` lines of file `f`. It is an implementation of tail -f n.
+  """
+  BUFSIZ = 1024
+  f.seek(0, 2)
+  bytes = f.tell()
+  size = lines + 1
+  block = -1
+  data = []
+  while size > 0 and bytes > 0:
+      if bytes - BUFSIZ > 0:
+	  # Seek back one whole BUFSIZ
+	  f.seek(block * BUFSIZ, 2)
+	  # read BUFFER
+	  data.insert(0, f.read(BUFSIZ))
+      else:
+	  # file too small, start from begining
+	  f.seek(0,0)
+	  # only read what was not read
+	  data.insert(0, f.read(bytes))
+      linesFound = data[0].count('\n')
+      size -= linesFound
+      bytes -= BUFSIZ
+      block -= 1
+  return string.join(''.join(data).splitlines()[-lines:], '\n')
+
+def isText(file):
+  """Return True if the mimetype of file is Text"""
+  if not os.path.exists(file):
+    return False
+  text_range = ''.join(map(chr, [7,8,9,10,12,13,27] + range(0x20, 0x100)))
+  is_binary_string = lambda bytes: bool(bytes.translate(None, text_range))
+  try:
+    return not is_binary_string(open(file).read(1024))
+  except:
+    return False
+  
+def md5sum(file):
+  """Compute md5sum of `file` and return hexdigest value"""
+  if os.path.isdir(file):
+    return False
+  try:
+    fh = open(file, 'rb')
+    m = hashlib.md5()
+    while True:
+      data = fh.read(8192)
+      if not data:
+	break
+      m.update(data)
+    return m.hexdigest()
+  except:
+    return False
\ No newline at end of file
